#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#define SECTOR_SIZE 512

typedef signed   char __s8;
typedef unsigned char __u8;
typedef signed   short __s16;
typedef unsigned short __u16;
typedef signed   long  __s32;
typedef unsigned long  __u32;
typedef unsigned long long __u64;

/* MSDOS partition table */
struct partition {
    __u8 boot_ind;		/* 0x80 - active */
    __u8 head;			/* starting head */
    __u8 sector;		/* starting sector */
    __u8 cyl;			/* starting cylinder */
    __u8 sys_ind;		/* What partition type */
    __u8 end_head;		/* end head */
    __u8 end_sector;		/* end sector */
    __u8 end_cyl;		/* end cylinder */
    __u32 start_sect;		/* starting sector counting from 0 */
    __u32 nr_sects;		/* nr of sectors in partition */
};

void
main(int argc, char **argv)
{
    char *device;
    int fd;
    unsigned char buf[SECTOR_SIZE];
    int n, i, j, c;

    if (argc != 2) {
	printf("Usage: %s <device>\n", argv[0]);
	exit(1);
    }

    device = argv[1];
    fd = open(device, O_RDONLY);
    if (lseek(fd, 0x1be, SEEK_CUR) < 0) {
	printf("Unable to seek to desired location\n");
	exit(1);
    }
    n = read(fd, buf, sizeof(struct partition)*4);
    if (n != sizeof(struct partition)*4) {
	printf("Unable to read partition table: %s\n", strerror(errno));
	exit(1);
    }
    for (c = 1, j = 0; j < n; j += sizeof(struct partition), c++) {
	printf("%d: ", c);
	for (i = j; i < j + 16; i += 2) {
	    printf("%02x%02x ", buf[i], buf[i+1]);
	}
	printf("\n");
    }
    close(fd);
}
